-- Crypto1 module
-- Author: Jiri Bucek, jiri.bucek@fit.cvut.cz
-- Application-specific details: Tomas Rosa, tomas.rosa@rb.cz
-- References:
-- [1] Flavio D. Garcia, Peter van Rossum, Roel Verdult, Ronny Wichers Schreur: Wirelessly Pickpocketing a Mifare Classic Card, To appear in IEEE Symposium on Security and Privacy (S&P 09), 2009.
-- [2] Tomas Rosa, personal communication, 2009

-- v 0.3 16.10.2009 (TR) -- filter function expansion, CLK_EN signal introduction
-- v 0.2 12.09.2009 (JB) -- deleted reset signal (unneeded)
-- v 0.1 11.09.2009 (JB) -- changed to serial data input, LFSR enable, input enable
-- v 0.0 09.09.2009 (JB) -- parallel initialization

library ieee;
use ieee.std_logic_1164.all;

entity CRYPTO1 is
		 port (
		 		 D: in std_logic;    -- input data to LFSR
		 		 L_EN: in std_logic;		 -- LFSR feedback enable
		 		 I_EN: in std_logic;		 -- LFSR input enable
		 		 CLK: in std_logic;
				 CLK_EN : in std_logic;
		 		 Y: out std_logic;		 -- output of filter function
				 DBGOut : out std_logic_vector(41 to 47)	-- debug output
		 );
end;

architecture RTL of CRYPTO1 is

function XOR_REDUCE (constant X: in std_logic_vector) return std_logic is
		 variable S : std_logic;
begin
		 S := '0';
		 for I in X'range loop
		 		 S := S xor X(I);
		 end loop;
		 return S;
end;

-- components of filter function (def. 2.2 in [1])
--function Fa (constant X: in std_logic_vector(0 to 3)) return std_logic is
--begin
--		 return (X(0) or X(1)) xor (X(0) and X(3)) xor (X(2) and ((X(0) xor X(1)) or X(3)));
--end;

-- !be aware!: the order of input variables is reversed with respect to
-- the previous definition of Fa (the one employing xor)
function Fa (constant x: in std_logic_vector(3 downto 0)) return std_logic is
  variable y : std_logic;
  variable t : std_logic_vector ( 1 to 6 );
begin
  t(1) := not( x(3) ) and not( x(2) ) and not( x(1) );
  t(2) := not( x(3) ) and not( x(2) ) and x(1) and not( x(0) );
  t(3) := not( x(3) ) and x(2) and x(1);
  t(4) := x(3) and not( x(2) ) and not( x(1) ) and x(0);
  t(5) := x(3) and not( x(2) ) and x(1) and not( x(0) );
  t(6) := x(3) and x(2) and not( x(1) ) and x(0);
  y := t(1) or t(2) or t(3) or t(4) or t(5) or t(6);
  return y;
end Fa;

--function Fb (constant X: in std_logic_vector(0 to 3)) return std_logic is
--begin
--		 return ((X(0) and X(1)) or X(2)) xor ((X(0) xor X(1)) and (X(2) or X(3)));
--end;

-- !be aware!: the order of input variables is reversed with respect to
-- the previous definition of Fb (the one employing xor)
function Fb (constant x: in std_logic_vector(3 downto 0)) return std_logic is
  variable y : std_logic;
  variable t : std_logic_vector ( 1 to 5 );
begin
  t(1) := not( x(3) ) and not( x(2) ) and not( x(1) );
  t(2) := not( x(3) ) and x(2) and not( x(1) ) and not( x(0) );
  t(3) := not( x(3) ) and x(2) and x(1);
  t(4) := x(3) and not( x(2) ) and not( x(1) ) and not( x(0) );
  t(5) := x(3) and not( x(2) ) and x(1);
  y := t(1) or t(2) or t(3) or t(4) or t(5);
  return y;
end Fb;

--function Fc (constant X: in std_logic_vector(0 to 4)) return std_logic is
--begin
--		 return (X(0) or ((X(1) or X(4)) and (X(3) xor X(4)))) xor ((X(0) xor (X(1) and X(3))) and ((X(2) xor X(3)) or (X(1) and X(4))));
--end;

-- !be aware!: the order of input variables is reversed with respect to
-- the previous definition of Fc (the one employing xor)
function Fc (constant x: in std_logic_vector(4 downto 0)) return std_logic is
  variable y : std_logic;
  variable t : std_logic_vector ( 1 to 11 );
begin
  t(1) := not( x(4) ) and not( x(3) ) and not( x(2) ) and not( x(1) );
  t(2) := not( x(4) ) and not( x(3) ) and x(2) and not( x(1) );
  t(3) := not( x(4) ) and not( x(3) ) and x(2) and x(1) and x(0);
  t(4) := not( x(4) ) and x(3) and not( x(2) ) and not( x(1) );
  t(5) := not( x(4) ) and x(3) and x(2) and x(1);
  t(6) := x(4) and not( x(3) ) and not( x(2) ) and not( x(1) );
  t(7) := x(4) and not( x(3) ) and not( x(2) ) and x(1) and not( x(0) );
  t(8) := x(4) and not( x(3) ) and x(2) and not( x(1) ) and not( x(0) );
  t(9) := x(4) and not( x(3) ) and x(2) and x(1) and not( x(0) );
  t(10) := x(4) and x(3) and not( x(2) ) and x(1) and not( x(0) );
  t(11) := x(4) and x(3) and x(2) and x(1) and not( x(0) );
  y := '0';
  for i in 1 to 11 loop
    y := y or t(i);
  end loop;
  return y;
end Fc;

-- LFSR xor feedback function
function L (constant X: in std_logic_vector(0 to 47)) return std_logic is
begin
		 return XOR_REDUCE(X(0)&X(5)&X(9)&X(10)&X(12)&X(14)&X(15)&X(17)&X(19)&X(24)&X(25)&X(27)&X(29)&X(35)&X(39)&X(41)&X(42)&X(43));
end L;

signal CTX: std_logic_vector(0 to 47); -- cipher context (content of shift register)
--signal C: std_logic_vector(0 to 4); -- internal signals of filter function

-- DEBUG: to see CTX as vector in post-fit simulation:
--attribute equivalent_register_removal: string;
--attribute equivalent_register_removal of CTX: signal is "yes"; --??

begin
		DBGOut <= CTX(41 to 47);	-- 7 msb bits for debug purpose

		 -- LFSR with single bit input with input enable and LFSR feedback enable
		 lfsr: process ( CLK )
		 begin
		 		 if CLK = '1' and CLK'event then
				   if ( CLK_EN = '1' ) then
		 		     CTX <= CTX(1 to 47) & ((L(CTX) and L_EN) xor (D and I_EN));
					else
					  CTX <= CTX;
               end if;					  
		 		 end if;
		 end process;
		 
		 -- filter function
		 filter: process ( CTX )
		   variable C : std_logic_vector(0 to 4);
		 begin
		   C(0) := Fa(CTX( 9) & CTX(11) & CTX (13) & CTX(15));
		   C(1) := Fb(CTX(17) & CTX(19) & CTX (21) & CTX(23));
		   C(2) := Fb(CTX(25) & CTX(27) & CTX (29) & CTX(31));
		   C(3) := Fa(CTX(33) & CTX(35) & CTX (37) & CTX(39));
		   C(4) := Fb(CTX(41) & CTX(43) & CTX (45) & CTX(47));
		 
		   Y <= Fc(C);
		end process;
		 
end;
