-- Crypto1 tester
-- Author: Jiri Bucek, jiri.bucek@fit.cvut.cz
-- Application-specific details: Tomas Rosa, tomas.rosa@rb.cz

-- v 0.4 9.11.2009 (TR) -- index handling corrected slightly in REV(...)
-- v 0.3 6.11.2009 (TR) -- configuration FSM support
-- v 0.2 12.09.2009 (JB) -- changed interface to 2-bit CMD, no reset
-- v 0.1 11.09.2009 (JB)

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity CRYPTO1_TEST is
end;

architecture TESTBENCH of CRYPTO1_TEST is

		 component CRYPTO1_TOP
		 		 port (
		 		 		 DIN : in std_logic;
		 		 		 DOUT : out std_logic;
		 		 		 CLK : in std_logic;
						 DBG_LFSR : out std_logic_vector(41 to 47);	-- drives LEDs (7)
						 DBG_LIF : out std_logic_vector(2 downto 0)	-- 7-sg. anodes
		 		 );
		 end component;
		 
		 signal		 CLK :  std_logic := '0';
		 signal		 DOUT :  std_logic;
		 signal		 DIN : std_logic;
		 signal		 LEDS : std_logic_vector(1 to 8);
		 signal		 DISP_ECG : std_logic_vector(2 downto 0);
		
		 function VectorToString(X : in std_logic_vector) return string is
			variable s : string (X'range);
			variable I : integer;
		 begin
		 	for I in X'left to X'right loop
				case X(I) is
				  when '1' => s(I) := '1';
				  when others => s(I) := '0';
				end case;
		 	end loop;
			return s;
		 end VectorToString;

		 procedure DoStream (variable X: inout std_logic_vector; signal INS: out std_logic)
		 is
		 begin
		 		 for I in X'low to X'high loop
						 report "X=" & VectorToString(X);
						 wait until CLK = '0';
						 wait for 15 ns;
		 		 		 INS <= X(I);
		 		 		 wait for 30 ns; -- wait for DOUT
		 		 		 X(I) := DOUT;
		 		 		 wait until CLK = '1';
		 		 		 wait for 15 ns;
		 		 end loop;
		 end;

		 procedure LoadKey (constant K: in std_logic_vector(0 to 47);
									signal INS: out std_logic) is
		 		 variable X : std_logic_vector(K'range);
		 begin
		 		 X := K;
		 		 DoStream(X, INS);
		 end;
		 
		 procedure ResetC1S( signal INS : inout std_logic ) is
		 begin
				 INS <= not INS;
				 wait until CLK = '0';
				 INS <= '0' after 15 ns;
				 wait until CLK = '1';
				 INS <= '1' after 15 ns;
		 end;

		 procedure NextC1S( signal INS : inout std_logic ) is
		 begin
				 INS <= not INS;
				 wait until CLK = '0';
				 INS <= '1' after 15 ns;
				 wait until CLK = '1';
				 INS <= '0' after 15 ns;
		 end;
	 
		 -- REV: convert left-to-right bytes with right-to-left bits to low-to-high bit stream, e.g. x"010F" to "1000000011110000"
		 function REV(X: in std_logic_vector) return std_logic_vector is
		 		 variable Y: std_logic_vector(X'low to X'high);
		 		 variable Z: std_logic_vector(X'low to X'high);
		 		 variable I, J, B, BLEN: integer;
		 begin
		 		 J:= X'high;
		 		 for I in X'left to X'right loop
		 		 		 Y(J) := X(I);
		 		 		 J := J - 1;
		 		 end loop;

		 		 BLEN := X'length/8;
				 J := X'low;
		 		 for I in 0 to BLEN-1 loop
		 		 		 Z(I*8+J to I*8+7+J) := Y((BLEN-I-1)*8+J to (BLEN-I-1)*8+7+J);
		 		 end loop;
		 		 return Z;
		 end REV;
		 
		 
begin
 		 uut: CRYPTO1_TOP port map ( DIN => DIN,
											  DOUT => DOUT,
											  CLK => CLK,
											  DBG_LFSR => LEDS(1 to 7),
											  DBG_LIF => DISP_ECG(2 downto 0) );

		 CLK <= not CLK after 50 ns;

		 tst: process
		 		 variable K : std_logic_vector(0 to 47);
		 		 variable X : std_logic_vector(0 to 31);
		 begin
		 		 DIN <= '0';
				 wait for 200 ns;
				 wait until CLK = '1';
				 wait for 15 ns;
				 ResetC1S( DIN );	 -- reset the config FSM, leave CLK ON
				 
		 		 -- Test vectors by Tomas Rosa
		 		 -- Test 1
		 		 K := REV(x"A0A1A2A3A4A5");
		 		 LoadKey(K, DIN);
--		 		 X := REV(x"C1C2C3C4" xor x"5672D263"); -- nt xor u
		 		 X := REV(x"BEBA93BE" xor x"5672D263"); -- BEBA test nt xor u
--		 		 X := x"C1C2C3C4" xor x"5672D263"; -- nt xor u
--				 X := REV( X );
--		 		 assert false report "xxxx debug stop xxxx" severity failure;
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
--		 		 X := REV(x"2DDFECA5"); --{nr}
		 		 X := REV(x"01020304"); -- BEBA test {nr}
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := (others => '0');
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
				 DoStream(X, DIN);	-- BEBA test (skip {a_T} CLKs)
		 		 assert X = REV(x"8C6CF5D9") report "Test error" severity failure;

		 		 -- Test 2
				 ResetC1S( DIN );	 -- reset the config FSM, leave CLK ON
		 		 K := REV(x"A0A1A2A3A4A5");
		 		 LoadKey(K, DIN);
		 		 X := REV(x"D1D2D3D4" xor x"5672D263"); -- nt xor u
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := REV(x"6C5EE065"); --{nr}
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := (others => '0');
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 assert X = REV(x"5CDEEAFE") report "Test error" severity failure;

--		 		 assert false report "DEBUG: End of simulation." severity failure;

		 		 -- Test 3
				 ResetC1S( DIN );	 -- reset the config FSM, leave CLK ON
		 		 K := REV(x"A0A1A2A3A4A5");
		 		 LoadKey(K, DIN);
		 		 X := REV(x"C1C2C3C4" xor x"5672D263"); -- nt xor u
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := REV(x"330EE975"); --{nr}
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := (others => '0');
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 assert X = REV(x"74F2687E") report "Test error" severity failure;

		 		 -- Test 4
				 ResetC1S( DIN );	 -- reset the config FSM, leave CLK ON
		 		 K := REV(x"A0A1A2A3A4A5");
		 		 LoadKey(K, DIN);
		 		 X := REV(x"D1D2D3D4" xor x"5672D263"); -- nt xor u
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := REV(x"EF9C1717"); --{nr}
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 X := (others => '0');
				 NextC1S( DIN );		-- next state of config FSM
		 		 DoStream(X, DIN);
		 		 assert X = REV(x"6B72E239") report "Test error" severity failure;

		 		 assert false report "End of simulation." severity failure;
		 end process;
		 
	 
end;
