-- Crypto1_TOP module
-- Author: Jiri Bucek, jiri.bucek@fit.cvut.cz
-- Application-specific details: Tomas Rosa, tomas.rosa@rb.cz

-- v 0.3 16.10.2009 (TR) -- configuration FSM
-- v 0.2 12.09.2009 (JB) -- changed interface to 2-bit CMD, no reset
-- v 0.1 10.09.2009 (JB)

library ieee;
use ieee.std_logic_1164.all;

entity CRYPTO1_TOP is
		 port (
		 		 DIN: in std_logic;
		 		 DOUT: out std_logic;
--				 CMD : in std_logic_vector(1 downto 0);
-- CMD encoding: "00" load, "01" mix, "10" decrypt and mix, "11" decrypt
		 		 CLK: in std_logic;
				 DBG_LFSR : out std_logic_vector(41 to 47);	-- drives LEDs (7)
				 DBG_LIF : out std_logic_vector(2 downto 0)	-- 7-sg. anodes
		 );
end;

architecture RTL of CRYPTO1_TOP is

		 component CRYPTO1
		 		 port (
		 		 		 D: in std_logic;			 -- data input
		 		 		 L_EN: in std_logic;		 -- LFSR feedback enable
		 		 		 I_EN: in std_logic;		 -- LFSR input enable
		 		 		 CLK: in std_logic;		 -- clock (posedge)
						 CLK_EN : in std_logic;  -- clock enable
						 Y: out std_logic;		 -- output of filter function
						 DBGOut : out std_logic_vector(41 to 47)	-- debug output
		 		 );
		 end component;

		 signal FEED, Y : std_logic;

		 signal L_EN: std_logic;		 		 -- LFSR feedback enable
		 signal I_EN: std_logic;		 		 -- input enable
		 signal F_EN: std_logic;		 		 -- filter function enable
		 
		 signal LIF : std_logic_vector(2 downto 0);
		 signal LIF_next : std_logic_vector(2 downto 0);
		 
		 type conf_state is ( CF_IDLE, CF_RUN );
		 signal cf_state : conf_state := CF_IDLE;
		 signal Crypto1_CLK_EN : std_logic;
		 signal DIN_sample : std_logic;
		 
begin
		 core: CRYPTO1 port map ( D => FEED,
							L_EN => L_EN, 
							I_EN => I_EN,
							CLK => CLK,
							CLK_EN => Crypto1_CLK_EN,
							Y => Y,
							DBGOut => DBG_LFSR );
		 
		 FEED <= DIN xor (Y and F_EN);
		 DOUT <= FEED;

--	    with CMD select
--		 		 LIF <= 
--		 		 		 "010" when "00", -- load key
--		 		 		 "110" when "01", -- mix uid xor n_T
--		 		 		 "111" when "10", -- decrypt and mix {n_R}
--		 		 		 "101" when others; -- decrypt data (without mixing)
		 (L_EN, I_EN, F_EN) <= LIF;
		 DBG_LIF <= LIF;
		 Crypto1_CLK_EN <= '1' when cf_state = CF_IDLE else '0';

       process ( CLK )	-- configuration FSM action
		 begin
		   if CLK = '1' and CLK'event then -- @posedge
			  if cf_state = CF_RUN then
			    LIF <= LIF_next;
			  end if;
			  DIN_sample <= DIN; 
			end if;
		 end process;
		 
		 process ( LIF, DIN ) -- configuration FSM - transition function
		 begin
		   case LIF is
			  when "010" =>
			    if ( DIN = '1' ) then
			      LIF_next <= "110";	-- mix with (UID xor n_T)
				 else
				   LIF_next <= "010";	-- reset: load key
				 end if;
			  when "110" =>
			    if ( DIN = '1' ) then
			      LIF_next <= "111";	-- decrypt {n_R} and mix
				 else
				   LIF_next <= "010";	-- reset: load key
				 end if;
			  when "111" =>
			    if ( DIN = '1' ) then
			      LIF_next <= "101";	-- decrypt data, do not mix
				 else
				   LIF_next <= "010";	-- reset: load key
				 end if;
			  when others =>
	          LIF_next <= "010";		-- reset: load key
			end case;
		 end process;
		 
		 process ( CLK ) -- DIN comparison, config startup/termination
		 begin
		   if CLK = '0' and CLK'event then -- @negedge
			  if cf_state /= CF_RUN and DIN_sample /= DIN then
				 cf_state <= CF_RUN;
			  end if;
			  if cf_state = CF_RUN and DIN_sample /= DIN then
				 cf_state <= CF_IDLE;
			  end if;
			end if;
		 end process;
		    
end;
